package datamodel;

import java.sql.*;
import java.util.ArrayList;
/**
 * Database is a class that specifies the interface to the 
 * movie database. Uses JDBC and the MySQL Connector/J driver.
 */
public class Database {
    /** 
     * The database connection.
     */
    private Connection conn;
        
    /**
     * Create the database interface object. Connection to the database
     * is performed later.
     */
    public Database() {
        conn = null;
    }
       
    /* --- TODO: Change this method to fit your choice of DBMS --- */
    /** 
     * Open a connection to the database, using the specified user name
     * and password.
     *
     * @param userName The user name.
     * @param password The user's password.
     * @return true if the connection succeeded, false if the supplied
     * user name and password were not recognized. Returns false also
     * if the JDBC driver isn't found.
     */
    public boolean openConnection(String userName, String password) {
        try {
        	// Connection strings for included DBMS clients:
        	// [MySQL]       jdbc:mysql://[host]/[database]
        	// [PostgreSQL]  jdbc:postgresql://[host]/[database]
        	// [SQLite]      jdbc:sqlite://[filepath]
        	
        	// Use "jdbc:mysql://puccini.cs.lth.se/" + userName if you using our shared server
        	// If outside, this statement will hang until timeout.
            conn = DriverManager.getConnection 
                ("jdbc:sqlite:movieBookings.db3", userName, password);
        }
        catch (SQLException e) {
            System.err.println(e);
            e.printStackTrace();
            return false;
        }
        return true;
    }

    // Reads a file from the resources folder and returns the content as a string
    private String readResourceFile(String fileName) {
        String content = "";
        try {
            content = new String(getClass().getResourceAsStream(fileName).readAllBytes());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return content;
    }

    public void runMigration() {
        // This path needs to start with a /
        String migration = readResourceFile("/migration.sql");
        try {
            Statement stmt = conn.createStatement();
            stmt.executeUpdate(migration);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }
        
    /**
     * Close the connection to the database.
     */
    public void closeConnection() {
        try {
            if (conn != null)
                conn.close();
        }
        catch (SQLException e) {
        	e.printStackTrace();
        }
        conn = null;
        
        System.err.println("Database connection closed.");
    }
        
    /**
     * Check if the connection to the database has been established
     *
     * @return true if the connection has been established
     */
    public boolean isConnected() {
        return conn != null;
    }
	
    // Gets the Show object with the given title and date
  	public Show getShowData(String mTitle, String mDate) {
		Integer mFreeSeats = 42;
		String mVenue = "Kino 2";

        if(!isConnected()) {
            System.err.println("getShowData: no connection to database");
            return null;
        }

        try {
            Statement stmt = conn.createStatement();
            // ResultSet rs = stmt.executeQuery("SELECT * FROM Showings WHERE title = '" + mTitle + "' AND date = '" + mDate + "'");
            ResultSet rs = stmt.executeQuery("SELECT *, Theaters.name AS theater_name FROM Showings JOIN Movies ON Showings.movie_id = Movies.movie_id JOIN Theaters ON Showings.theater_id = Theaters.theater_id");
            if (rs.next()) {
                mFreeSeats = rs.getInt("total_seats");
                mVenue = rs.getString("theater_name");
            } else {
                System.err.println("getShowData: no show found");
                return null;
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
		
		return new Show(mTitle, mDate, mVenue, mFreeSeats);
	}

    public ArrayList<Show> getAllShows() {
        ArrayList<Show> shows = new ArrayList<Show>();

        if(!isConnected()) {
            System.err.println("getAllShows: no connection to database");
            return shows;
        }

        try {
            Statement stmt = conn.createStatement();
            ResultSet rs = stmt.executeQuery("SELECT *, Theaters.name AS theater_name FROM Showings JOIN Movies ON Showings.movie_id = Movies.movie_id JOIN Theaters ON Showings.theater_id = Theaters.theater_id");
            while (rs.next()) {
                String mTitle = rs.getString("name");
                String mDate = rs.getString("show_date");
                Integer mFreeSeats = rs.getInt("total_seats");
                String mVenue = rs.getString("theater_name");
                shows.add(new Show(mTitle, mDate, mVenue, mFreeSeats));
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return shows;
    }

    public ArrayList<String> getDatesForMovie(String mTitle) {
        ArrayList<String> dates = new ArrayList<String>();

        if(!isConnected()) {
            System.err.println("getDatesForMovie: no connection to database");
            return dates;
        }

        try {
            Statement stmt = conn.createStatement();
            ResultSet rs = stmt.executeQuery("SELECT show_date FROM Showings JOIN Movies ON Showings.movie_id = Movies.movie_id WHERE name = '" + mTitle + "'");
            while (rs.next()) {
                dates.add(rs.getString("show_date"));
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return dates;
    }

    public boolean login(String username) {
        if(!isConnected()) {
            System.err.println("login: no connection to database");
            return false;
        }

        try {
            Statement stmt = conn.createStatement();
            ResultSet rs = stmt.executeQuery("SELECT * FROM users WHERE username = '" + username + "'");
            if (rs.next()) {
                return true;
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return false;
    }

    /* --- TODO: insert more own code here --- */
}
